package gs.smolban.db

import gs.slug.v0.Slug
import gs.smolban.model.Group
import gs.smolban.model.SmolbanError
import gs.smolban.model.Ticket

/** Parent type of all database errors in Smolban.
  */
sealed trait DbError extends SmolbanError

object DbError:

  /** Returned if an attempt is made to insert a [[Ticket]], where the Ticket ID
    * already exists for the specified [[gs.smolban.model.Group]].
    *
    * @param ref
    *   The Ticket Reference that was in conflict.
    */
  case class TicketAlreadyExists(ref: Ticket.Reference) extends DbError

  case class GroupAlreadyExists(
    id: Group.Id,
    slug: Slug
  ) extends DbError

end DbError
