package gs.smolban.db

import cats.data.EitherT
import gs.smolban.model.Group

/** Database interface for [[Group]].
  */
trait GroupDb[F[_]]:
  /** Create a new [[Group]] in the database.
    *
    * @param group
    *   The [[Group]] to store.
    * @return
    *   The [[Group]] that was stored, or an error.
    */
  def createGroup(group: Group): EitherT[F, DbError, Group]

  /** Read the specified [[Group]] from the database.
    *
    * @param id
    *   The unique identifier of the [[Group]] to retrieve.
    * @return
    *   The [[Group]], or `None` if the specified ID does not exist.
    */
  def readGroup(id: Group.Id): F[Option[Group]]

  /** Delete the specified [[Group]]. Note that this operation deletes all
    * [[Ticket]] associated with the group and should be used with care.
    *
    * @param id
    *   The unique identifier of the [[Group]] to delete.
    * @return
    *   `true` if the group was deleted, `false` otherwise.
    */
  def deleteGroup(id: Group.Id): F[Boolean]
