package gs.smolban.db.doobie

import DoobieTypes.ErrorMessages
import doobie.*
import gs.slug.v0.Slug
import gs.smolban.model.CreatedAt
import gs.smolban.model.Group
import gs.smolban.model.Ticket
import gs.uuid.v0.UUID
import gs.uuid.v0.UUIDFormat

trait DoobieTypes:

  implicit val ticketIdGet: Get[Ticket.Id] =
    Get[Long].tmap(Ticket.Id(_))

  implicit val ticketIdPut: Put[Ticket.Id] =
    Put[Long].tcontramap(_.toLong())

  implicit val ticketStatusGet: Get[Ticket.Status] =
    Get[String].temap(dbValue =>
      Ticket.Status.parse(dbValue) match
        case None         => Left(ErrorMessages.invalidTicketStatus(dbValue))
        case Some(status) => Right(status)
    )

  implicit val ticketStatusPut: Put[Ticket.Status] =
    Put[String].tcontramap(_.name)

  implicit val groupIdGet: Get[Group.Id] =
    Get[Array[Byte]].tmap(bytes => Group.Id(UUID(UUIDFormat.fromBytes(bytes))))

  implicit val groupIdPut: Put[Group.Id] =
    Put[Array[Byte]].tcontramap(id => UUIDFormat.toBytes(id.toUUID().toUUID()))

  implicit val createdAtGet: Get[CreatedAt] =
    Get[Long].tmap(CreatedAt.fromMilliseconds)

  implicit val createdAtPut: Put[CreatedAt] =
    Put[Long].tcontramap(_.toMilliseconds())

  implicit val slugGet: Get[Slug] =
    Get[String].temap { slug =>
      Slug.validate(slug) match
        case None    => Left(ErrorMessages.invalidSlug(slug))
        case Some(s) => Right(s)
    }

  implicit val slugPut: Put[Slug] =
    Put[String].tcontramap(_.str())

object DoobieTypes extends DoobieTypes:

  object ErrorMessages:

    def invalidTicketStatus(candidate: String): String =
      s"'$candidate' is not a valid ticket status."

    def invalidSlug(candidate: String): String =
      s"'$candidate' is not a valid Group slug."

  end ErrorMessages

end DoobieTypes
