package gs.smolban.model

import cats.Show
import java.time.Instant

/** Describes an instant at which something was created. Opaque type for
  * `java.time.Instant`.
  */
opaque type CreatedAt = Instant

object CreatedAt:

  /** Instantiate a new [[CreatedAt]].
    *
    * @param timestamp
    *   The underlying timestamp.
    * @return
    *   The new instance.
    */
  def apply(timestamp: Instant): CreatedAt = timestamp

  /** Instantiate a new [[CreatedAt]].
    *
    * @param millis
    *   The epoch milliseconds describing the instant of this timestamp.
    * @return
    *   The new instance.
    */
  def fromMilliseconds(millis: Long): CreatedAt = Instant.ofEpochMilli(millis)

  given CanEqual[CreatedAt, CreatedAt] = CanEqual.derived

  given Show[CreatedAt] = _.toInstant().toString()

  extension (createdAt: CreatedAt)
    /** Unwrap this value.
      *
      * @return
      *   The underlying `Instant` value.
      */
    def toInstant(): Instant = createdAt

    /** Express this instant as epoch milliseconds.
      *
      * @return
      *   The epoch milliseconds.
      */
    def toMilliseconds(): Long = createdAt.toEpochMilli()

end CreatedAt
